@@ -3,6 +3,8 @@
 
 /// Errors that can occur during sending/receiving of [`Command`]s and [`Event`]s.
 #[derive(Debug, thiserror::Error)]
+// TODO
+#[allow(clippy::enum_variant_names)]
 pub enum Error {
     /// A command could not be sent.
     #[error("Error sending command.")]
@@ -16,7 +18,7 @@ pub enum Error {
     #[error("Error receiving event.")]
     ReceivingEventFailed,
 
-    /// A peer error occured
-    #[error("{:?}", 0)]
+    /// An error regarding a specific peer occured.
+    #[error("{0}")]
     PeerError(#[from] crate::peer::error::Error),
 }
