@@ -1,4 +1,4 @@
 // Copyright 2020-2021 IOTA Stiftung
 // SPDX-License-Identifier: Apache-2.0
 
-pub mod gossip;
+pub mod iota_gossip;
